﻿using System;
using System.Drawing;
using System.Windows.Forms;
using System.Collections.Generic;


namespace WindowsFormsApplication1
{
    public enum Side { Top, Bottom, Right, Left };

    
    public enum Corner { NE, NW, SE, SW };
    class Positioning
    {


        // some utils.
        static int RightBounds(Form f)
        {
            Rectangle desktop = Screen.PrimaryScreen.WorkingArea;
            return desktop.Right - f.Width;
        }
        static int LeftBounds()
        {
            return 0;
        }
        static int BottomBounds(Form f)
        {
            Rectangle desktop = Screen.PrimaryScreen.WorkingArea;
            return desktop.Bottom - f.Height;
        }
        static int TopBounds()
        {
            return 0;
        }

        static int FormLeft(Form f)
        { return f.Location.X; }
        static int FormRight(Form f)
        {
        return FormLeft(f)+f.Width;
        }
        static int FormUp(Form f)
        {
        return f.Location.Y;
        }
        static int FormDown(Form f)
        { return FormUp(f) + f.Height ; }

    // almost unused
    public static void moveToCorner(Corner corner, Form f)
    {
        Rectangle desktop = Screen.PrimaryScreen.WorkingArea;
        int x = 0;
        int y = 0;
        switch (corner)
        {
            case Corner.NE:
                y = TopBounds();
                x = RightBounds(f);
                break;
            case Corner.NW:
                y = TopBounds();
                x = LeftBounds();
                break;

            case Corner.SE:
                x = RightBounds(f);
                y = BottomBounds(f);
                break;
            case Corner.SW:
                x = LeftBounds();
                y = BottomBounds(f);
                break;
        }
        f.SetDesktopLocation(x, y);
     }


        
        static List<Pair<Side, int>> DistanceFromWalls(Form f)
        {
            var distances = new List<Pair<Side, int>>();
            Rectangle desktop = Screen.PrimaryScreen.WorkingArea;

            distances.Add(new Pair<Side, int>(Side.Right, Math.Abs( FormRight(f) - desktop.Right)));
            distances.Add(new Pair<Side, int>(Side.Bottom,Math.Abs(  FormDown(f) - desktop.Bottom))); 
            distances.Add(new Pair<Side, int>(Side.Left, Math.Abs( FormLeft(f) - desktop.Left)));
            distances.Add(new Pair<Side, int>(Side.Top, Math.Abs(FormUp(f) - desktop.Top)));

            return distances;

            
        }
        static bool AtWall(Form f, out Side[] sides)
        { 
            var dists = DistanceFromWalls(f);
            List<Side> atwalls = dists.FindAll(x => x.second == 0).ConvertAll(x=> x.first);
            // finds all pairs where the distance is 0, retain only the side from the pairs found .
            
            sides = atwalls.ToArray();
            return (atwalls.Count != 0); // if found something, then we're at wall.
        }

        static bool NearWall(Form f, out Side[] nearWalls)
        {
            List<Side> list = new List<Side>();
            Rectangle desktop = Screen.PrimaryScreen.WorkingArea;
            int x = (int)(desktop.Width * 0.02);
            
            int y = (int)(desktop.Height * 0.02);

        //    MessageBox.Show(x + " " + y);

            var distances = DistanceFromWalls(f);
            foreach (Pair<Side, int> pair in distances)
            {
                if ((pair.first == Side.Bottom) || (pair.first == Side.Top))
                {
                    if (pair.second < y && pair.second != 0) { list.Add(pair.first); }
                }
                if ((pair.first == Side.Right) || (pair.first == Side.Left))
                {
                    if (pair.second < x && pair.second != 0) { list.Add(pair.first); }
                }
            }
            nearWalls = list.ToArray();
            return (list.Count!=0);
        }
        public static void JumpWalls(Form f, Side[] s)
        {
            
            foreach (Side a in s)
            {
                Point p = f.Location;
                switch (a)
                {
                    case Side.Left:         
                        p.X = LeftBounds();
                        break;
                    case Side.Right:
                        p.X = RightBounds(f);
                        break;
                    case Side.Top:
                        p.Y = TopBounds();
                        break;
                    case Side.Bottom:
                        p.Y = BottomBounds(f);
                        break;

                }
                f.Location = p;
            }
        }

        public static bool outside(Form f, out Side[] x)
        {
            List<Side> sides =  new List<Side>();
            Rectangle desktop = Screen.PrimaryScreen.WorkingArea;
             if (FormLeft(f) < desktop.Left) 
                 sides.Add(Side.Left);
            if  (FormUp(f) < desktop.Top)
                sides.Add(Side.Top);
            if (FormRight(f) > desktop.Right)
                sides.Add(Side.Right);
            if (FormDown(f) > desktop.Bottom)
                sides.Add(Side.Bottom);

            x = sides.ToArray();
            return (sides.Count!=0);
        }
            public static void Magnetize(Form f)
            {
                Side[] nearsides = null; bool isNear = (NearWall(f, out nearsides));
            
                Side[] outsides = null;
                bool rev = outside(f, out outsides); // if outside of screen

                if (rev)
                {
                    JumpWalls(f, outsides); // jump walls and recalculate
                    isNear = (NearWall(f, out nearsides)); 
                }
           
            // Side[] walls = null; bool atw = AtWall(f, out walls);
            if (isNear) // still near
                JumpWalls(f,nearsides);
            }
    }
}
